@@ -74,6 +74,7 @@ typedef struct
   int plyr[8];
   int trail_1;
   int trail_2;
+  int pickup;
 } mapcolor_t;
 
 typedef struct map_point_s
@@ -166,6 +167,7 @@ typedef enum
 #if defined(HAVE_LIBSDL2_IMAGE)
   map_things_appearance_icon,
 #endif
+  map_things_appearance_box,
 
   map_things_appearance_max
 } map_things_appearance_t;
