@@ -18,6 +18,7 @@
 #include <string.h>
 
 #include "am_map.h"
+#include "d_deh.h"
 #include "doomdef.h"
 #include "doomstat.h"
 #include "hu_stuff.h"
@@ -27,6 +28,7 @@
 #include "r_main.h"
 #include "r_segs.h"
 #include "s_sound.h"
+#include "st_stuff.h"
 #include "smooth.h"
 #include "v_video.h"
 #include "z_zone.h"
@@ -109,7 +111,7 @@ void S_Init(void);
 void M_ChangeMIDIPlayer(void);
 void HU_InitCrosshair(void);
 void HU_InitThresholds(void);
-void dsda_InitKeyFrame(void);
+void dsda_InitAutoKeyFrames(void);
 void dsda_SetupStretchParams(void);
 void dsda_InitCommandHistory(void);
 void dsda_InitQuickstartCache(void);
@@ -133,6 +135,8 @@ void dsda_InitExHud(void);
 void dsda_UpdateFreeText(void);
 void dsda_ResetAirControl(void);
 void dsda_AlterGameFlags(void);
+void dsda_RefreshPistolStart(void);
+void dsda_RefreshAlwaysPistolStart(void);
 
 void dsda_TrackConfigFeatures(void) {
   if (!demorecording)
@@ -194,6 +198,12 @@ void dsda_TrackConfigFeatures(void) {
 
   if (dsda_IntConfig(dsda_config_map_textured) || dsda_IntConfig(dsda_config_show_minimap))
     dsda_TrackFeature(uf_advanced_map);
+
+  if (dsda_IntConfig(dsda_config_translucent_sprites) > 1)
+    dsda_TrackFeature(uf_vanillatrans);
+
+  if (dsda_IntConfig(dsda_config_translucent_ghosts))
+    dsda_TrackFeature(uf_ghosttrans);
 }
 
 // TODO: migrate all kinds of stuff from M_Init
@@ -202,7 +212,7 @@ void dsda_TrackConfigFeatures(void) {
 void dsda_UpdateStrictMode(void) {
   I_Init2(); // side effect of realtic clock rate
   M_ChangeSpeed(); // side effect of always sr50
-  dsda_InitKeyFrame();
+  dsda_InitAutoKeyFrames();
   M_ChangeSkyMode(); // affected by mouselook setting
   HU_InitCrosshair();
   M_ChangeApplyPalette();
@@ -312,6 +322,10 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "sts_traditional_keys", dsda_config_sts_traditional_keys,
     CONF_BOOL(0), &sts_traditional_keys
   },
+  [dsda_config_sts_solid_bg_color] = {
+    "sts_solid_bg_color", dsda_config_sts_solid_bg_color,
+    CONF_BOOL(0), NULL, NOT_STRICT, ST_SetResolution
+  },
   [dsda_config_strict_mode] = {
     "dsda_strict_mode", dsda_config_strict_mode,
     CONF_BOOL(1), NULL, NOT_STRICT, dsda_UpdateStrictMode
@@ -562,7 +576,7 @@ dsda_config_t dsda_config[dsda_config_count] = {
   },
   [dsda_config_mapcolor_sprt] = {
     "mapcolor_sprt", dsda_config_mapcolor_sprt,
-    CONF_COLOR(112), &mapcolor.sprt
+    CONF_COLOR(88), &mapcolor.sprt
   },
   [dsda_config_mapcolor_item] = {
     "mapcolor_item", dsda_config_mapcolor_item,
@@ -596,6 +610,10 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "mapcolor_trail_2", dsda_config_mapcolor_trail_2,
     CONF_COLOR(100), &mapcolor.trail_2
   },
+  [dsda_config_mapcolor_pickup] = {
+    "mapcolor_pickup", dsda_config_mapcolor_pickup,
+    CONF_COLOR(112), &mapcolor.pickup
+  },
   [dsda_config_gl_skymode] = {
     "gl_skymode", dsda_config_gl_skymode,
     dsda_config_int, skytype_auto, skytype_count - 1, { skytype_auto }, NULL,
@@ -807,7 +825,7 @@ dsda_config_t dsda_config[dsda_config_count] = {
   },
   [dsda_config_cap_muxcommand] = {
     "cap_muxcommand", dsda_config_cap_muxcommand,
-    CONF_STRING("ffmpeg -i temp_v.nut -i temp_a.nut -c copy -y %f")
+    CONF_STRING("ffmpeg -i temp_v.nut -i temp_a.nut -r %r -c copy -y %f")
   },
   [dsda_config_cap_tempfile1] = {
     "cap_tempfile1", dsda_config_cap_tempfile1,
@@ -895,15 +913,15 @@ dsda_config_t dsda_config[dsda_config_count] = {
   },
   [dsda_config_auto_key_frame_interval] = {
     "dsda_auto_key_frame_interval", dsda_config_auto_key_frame_interval,
-    dsda_config_int, 1, 600, { 1 }, NULL, STRICT_INT(1), dsda_InitKeyFrame
+    dsda_config_int, 1, 600, { 1 }, NULL, STRICT_INT(1), dsda_InitAutoKeyFrames
   },
   [dsda_config_auto_key_frame_depth] = {
     "dsda_auto_key_frame_depth", dsda_config_auto_key_frame_depth,
-    dsda_config_int, 0, 600, { 60 }, NULL, STRICT_INT(0), dsda_InitKeyFrame
+    dsda_config_int, 0, 600, { 60 }, NULL, STRICT_INT(0), dsda_InitAutoKeyFrames
   },
   [dsda_config_auto_key_frame_timeout] = {
     "dsda_auto_key_frame_timeout", dsda_config_auto_key_frame_timeout,
-    dsda_config_int, 0, 25, { 10 }, NULL, STRICT_INT(0), dsda_InitKeyFrame
+    dsda_config_int, 0, 25, { 10 }, NULL, STRICT_INT(0), dsda_InitAutoKeyFrames
   },
   [dsda_config_auto_save] = {
     "dsda_config_auto_save", dsda_config_auto_save,
@@ -971,7 +989,11 @@ dsda_config_t dsda_config[dsda_config_count] = {
   },
   [dsda_config_pistol_start] = {
     "dsda_pistol_start", dsda_config_pistol_start,
-    CONF_BOOL(0), NULL, NOT_STRICT
+    CONF_BOOL(0), NULL, NOT_STRICT, dsda_RefreshPistolStart
+  },
+  [dsda_config_always_pistol_start] = {
+    "dsda_always_pistol_start", dsda_config_always_pistol_start,
+    CONF_BOOL(0), NULL, NOT_STRICT, dsda_RefreshAlwaysPistolStart
   },
   [dsda_config_respawn_monsters] = {
     "dsda_respawn_monsters", dsda_config_respawn_monsters,
@@ -1010,17 +1032,21 @@ dsda_config_t dsda_config[dsda_config_count] = {
     CONF_BOOL(1), NULL, STRICT_INT(1)
   },
   [dsda_config_viewbob] = {
-    "dsda_viewbob", dsda_config_viewbob,
-    CONF_BOOL(1)
+    "dsda_viewbob_pct", dsda_config_viewbob,
+    dsda_config_int, 0, 4, { 4 }
   },
   [dsda_config_weaponbob] = {
-    "dsda_weaponbob", dsda_config_weaponbob,
-    CONF_BOOL(1)
+    "dsda_weaponbob_pct", dsda_config_weaponbob,
+    dsda_config_int, 0, 4, { 4 }
   },
   [dsda_config_quake_intensity] = {
     "dsda_quake_intensity", dsda_config_quake_intensity,
     dsda_config_int, 0, 100, { 100 }
   },
+  [dsda_config_demo_end_quit] = {
+    "dsda_demo_end_quit", dsda_config_demo_end_quit,
+    CONF_BOOL(1)
+  },
   [dsda_config_map_blinking_locks] = {
     "map_blinking_locks", dsda_config_map_blinking_locks,
     CONF_BOOL(0), NULL, NOT_STRICT, AM_InitParams
@@ -1232,9 +1258,13 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "gl_fade_mode", dsda_config_gl_fade_mode,
     dsda_config_int, 0, 1, { 0 }
   },
-  [dsda_config_boom_translucent_sprites] = {
-    "boom_translucent_sprites", dsda_config_boom_translucent_sprites,
-    CONF_BOOL(1), NULL, NOT_STRICT, deh_changeCompTranslucency
+  [dsda_config_translucent_sprites] = {
+    "boom_translucent_sprites", dsda_config_translucent_sprites,
+    dsda_config_int, 0, 2, { 1 }, NULL, STRICT_INT(1), deh_changeCompTranslucency
+  },
+  [dsda_config_translucent_ghosts] = {
+    "translucent_ghosts", dsda_config_translucent_ghosts,
+    CONF_BOOL(0), NULL, STRICT_INT(0)
   },
   [dsda_config_show_alive_monsters] = { // never persisted
     "show_alive_monsters", dsda_config_show_alive_monsters,
@@ -1296,6 +1326,14 @@ dsda_config_t dsda_config[dsda_config_count] = {
     "announce_map", dsda_config_announce_map,
     CONF_BOOL(0),
   },
+  [dsda_config_extra_level_brightness] = {
+    "extra_level_brightness", dsda_config_extra_level_brightness,
+    dsda_config_int, 0, 4, {0}, NULL, STRICT_INT(0)
+  },
+  [dsda_config_playback_mouse_controls] = {
+    "playback_mouse_controls", dsda_config_playback_mouse_controls,
+    CONF_BOOL(1)
+  },
 };
 
 static void dsda_PersistIntConfig(dsda_config_t* conf) {
