@@ -48,6 +48,7 @@ typedef enum {
   dsda_config_sts_colored_numbers,
   dsda_config_sts_pct_always_gray,
   dsda_config_sts_traditional_keys,
+  dsda_config_sts_solid_bg_color,
   dsda_config_strict_mode,
   dsda_config_vertmouse,
   dsda_config_freelook,
@@ -119,6 +120,7 @@ typedef enum {
   dsda_config_mapcolor_frnd,
   dsda_config_mapcolor_trail_1,
   dsda_config_mapcolor_trail_2,
+  dsda_config_mapcolor_pickup,
   dsda_config_gl_skymode,
   dsda_config_gl_render_multisampling,
   dsda_config_gl_render_fov,
@@ -193,6 +195,7 @@ typedef enum {
   dsda_config_hud_health_green,
   dsda_config_hud_ammo_red,
   dsda_config_hud_ammo_yellow,
+  dsda_config_playback_mouse_controls,
   dsda_config_cycle_ghost_colors,
   dsda_config_auto_key_frame_interval,
   dsda_config_auto_key_frame_depth,
@@ -213,6 +216,7 @@ typedef enum {
   dsda_config_quickstart_cache_tics,
   dsda_config_death_use_action,
   dsda_config_allow_jumping,
+  dsda_config_always_pistol_start,
   dsda_config_pistol_start,
   dsda_config_respawn_monsters,
   dsda_config_fast_monsters,
@@ -277,7 +281,8 @@ typedef enum {
   dsda_config_render_stretchsky,
   dsda_config_render_linearsky,
   dsda_config_aspect_ratio_correction,
-  dsda_config_boom_translucent_sprites,
+  dsda_config_translucent_sprites,
+  dsda_config_translucent_ghosts,
   dsda_config_show_alive_monsters,
   dsda_config_left_analog_deadzone,
   dsda_config_right_analog_deadzone,
@@ -293,6 +298,8 @@ typedef enum {
   dsda_config_ansi_endoom,
   dsda_config_quit_sounds,
   dsda_config_announce_map,
+  dsda_config_extra_level_brightness,
+  dsda_config_demo_end_quit,
   dsda_config_count,
 } dsda_config_identifier_t;
 
