@@ -46,13 +46,14 @@
 
 #define INITIAL_DEMO_BUFFER_SIZE 0x20000
 
+int demo_tics;
+
 static char* dsda_demo_name_base;
 static byte* dsda_demo_write_buffer;
 static byte* dsda_demo_write_buffer_p;
 static int dsda_demo_write_buffer_length;
 static int dsda_extra_demo_header_data_offset;
 static int largest_real_offset;
-static int demo_tics;
 static int compatibility_level_unspecified;
 
 #define DSDA_UDMF_VERSION 1
@@ -277,7 +278,7 @@ void dsda_InitDemoRecording(void) {
     I_Error("You must specify a skill level when recording a demo!\n"
             "Example: dsda-doom -iwad DOOM -complevel 3 -skill 4 -record demo");
 
-  if (dsda_Flag(dsda_arg_pistolstart))
+  if (dsda_Flag(dsda_arg_pistol_start))
     I_Error("The -pistolstart option is not allowed when recording a demo!");
 
   demorecording = true;
@@ -295,7 +296,7 @@ void dsda_InitDemoRecording(void) {
   dsda_TrackConfigFeatures();
 
   if (!demo_key_frame_initialized) {
-    dsda_InitKeyFrame();
+    dsda_InitAutoKeyFrames();
     demo_key_frame_initialized = true;
   }
 
@@ -511,7 +512,7 @@ static dboolean dsda_UseDemoNameWithTime(void) {
 static char* dsda_DemoNameWithTime(void) {
   char* demo_name;
   char* base_name;
-  int counter = 2;
+  unsigned int counter = 2;
   size_t length;
 
   length = strlen(dsda_demo_name_base) + 16 + 1;
@@ -579,7 +580,7 @@ void dsda_EndDemoRecording(void) {
 void dsda_ExportDemo(const char* name) {
   char* demo_name;
   char* base_name;
-  int counter = 2;
+  unsigned int counter = 2;
   int old_offset;
 
   base_name = Z_Strdup(name);
