@@ -18,9 +18,8 @@
 #ifndef __DSDA_HUD_COMPONENT_BIG_ARMOR__
 #define __DSDA_HUD_COMPONENT_BIG_ARMOR__
 
-void dsda_InitBigArmorHC(int x_offset, int y_offset, int vpt_flags);
-void dsda_UpdateBigArmorHC(void);
-void dsda_DrawBigArmorHC(void);
-void dsda_EraseBigArmorHC(void);
+void dsda_InitBigArmorHC(int x_offset, int y_offset, int vpt_flags, int* args, int arg_count, void** data);
+void dsda_UpdateBigArmorHC(void* data);
+void dsda_DrawBigArmorHC(void* data);
 
 #endif
