@@ -18,9 +18,8 @@
 #ifndef __DSDA_HUD_COMPONENT_COMPOSITE_TIME__
 #define __DSDA_HUD_COMPONENT_COMPOSITE_TIME__
 
-void dsda_InitCompositeTimeHC(int x_offset, int y_offset, int vpt_flags);
-void dsda_UpdateCompositeTimeHC(void);
-void dsda_DrawCompositeTimeHC(void);
-void dsda_EraseCompositeTimeHC(void);
+void dsda_InitCompositeTimeHC(int x_offset, int y_offset, int vpt_flags, int* args, int arg_count, void** data);
+void dsda_UpdateCompositeTimeHC(void* data);
+void dsda_DrawCompositeTimeHC(void* data);
 
 #endif
