@@ -18,9 +18,8 @@
 #ifndef __DSDA_HUD_COMPONENT_HEALTH_TEXT__
 #define __DSDA_HUD_COMPONENT_HEALTH_TEXT__
 
-void dsda_InitHealthTextHC(int x_offset, int y_offset, int vpt_flags);
-void dsda_UpdateHealthTextHC(void);
-void dsda_DrawHealthTextHC(void);
-void dsda_EraseHealthTextHC(void);
+void dsda_InitHealthTextHC(int x_offset, int y_offset, int vpt_flags, int* args, int arg_count, void** data);
+void dsda_UpdateHealthTextHC(void* data);
+void dsda_DrawHealthTextHC(void* data);
 
 #endif
