@@ -18,9 +18,8 @@
 #ifndef __DSDA_HUD_COMPONENT_RENDER_STATS__
 #define __DSDA_HUD_COMPONENT_RENDER_STATS__
 
-void dsda_InitRenderStatsHC(int x_offset, int y_offset, int vpt_flags);
-void dsda_UpdateRenderStatsHC(void);
-void dsda_DrawRenderStatsHC(void);
-void dsda_EraseRenderStatsHC(void);
+void dsda_InitRenderStatsHC(int x_offset, int y_offset, int vpt_flags, int* args, int arg_count, void** data);
+void dsda_UpdateRenderStatsHC(void* data);
+void dsda_DrawRenderStatsHC(void* data);
 
 #endif
