@@ -18,9 +18,8 @@
 #ifndef __DSDA_HUD_COMPONENT_WEAPON_TEXT__
 #define __DSDA_HUD_COMPONENT_WEAPON_TEXT__
 
-void dsda_InitWeaponTextHC(int x_offset, int y_offset, int vpt_flags);
-void dsda_UpdateWeaponTextHC(void);
-void dsda_DrawWeaponTextHC(void);
-void dsda_EraseWeaponTextHC(void);
+void dsda_InitWeaponTextHC(int x_offset, int y_offset, int vpt_flags, int* args, int arg_count, void** data);
+void dsda_UpdateWeaponTextHC(void* data);
+void dsda_DrawWeaponTextHC(void* data);
 
 #endif
