@@ -56,11 +56,13 @@ static int auto_kf_timeout_count;
 #define TIMEOUT_LIMIT 1
 
 static dsda_key_frame_t first_kf;
-static dsda_key_frame_t quick_kf;
 static dsda_key_frame_t temp_kf;
-static auto_kf_t* auto_key_frames;
+dsda_key_frame_t quick_kf;
+auto_kf_t* auto_key_frames;
+int auto_kf_size = 0;
 static auto_kf_t* last_auto_kf;
-static int auto_kf_size;
+dsda_key_frame_t* playback_key_frames;
+int playback_kf_size = 0;
 static int restore_key_frame_index = -1;
 
 static int dsda_auto_key_frame_interval;
@@ -133,17 +135,17 @@ static void dsda_RewindKF(auto_kf_t** current) {
 static dsda_key_frame_t* dsda_ClosestKeyFrame(int target_tic_count) {
   dsda_key_frame_t* closest = NULL;
 
-  if (last_auto_kf) {
-    auto_kf_t* auto_kf;
+  if (auto_key_frames)
+    for (int i = 0; i < auto_kf_size; i++)
+      if (auto_key_frames[i].kf.game_tic_count <= target_tic_count)
+        if (!closest || auto_key_frames[i].kf.game_tic_count > closest->game_tic_count)
+          closest = &auto_key_frames[i].kf;
 
-    auto_kf = last_auto_kf;
-    for (auto_kf = last_auto_kf; auto_kf && auto_kf->kf.buffer; dsda_RewindKF(&auto_kf))
-      if (auto_kf->kf.game_tic_count <= target_tic_count)
-        if (!closest || auto_kf->kf.game_tic_count > closest->game_tic_count) {
-          closest = &auto_kf->kf;
-          break;
-        }
-  }
+  if (playback_key_frames)
+    for (int i = 0; i < playback_kf_size; i++)
+      if (playback_key_frames[i].game_tic_count <= target_tic_count)
+        if (!closest || playback_key_frames[i].game_tic_count > closest->game_tic_count)
+          closest = &playback_key_frames[i];
 
   if (!demorecording && temp_kf.buffer)
     if (temp_kf.game_tic_count <= target_tic_count)
@@ -169,7 +171,7 @@ void dsda_CopyKeyFrame(dsda_key_frame_t* dest, dsda_key_frame_t* source) {
   memcpy(dest->buffer, source->buffer, dest->buffer_length);
 }
 
-void dsda_InitKeyFrame(void) {
+void dsda_InitAutoKeyFrames(void) {
   int i;
 
   dsda_auto_key_frame_interval = dsda_IntConfig(dsda_config_auto_key_frame_interval);
@@ -183,11 +185,11 @@ void dsda_InitKeyFrame(void) {
     return;
   }
 
+  ++auto_kf_size; // chain includes a terminator
+
   if (auto_key_frames != NULL)
     Z_Free(auto_key_frames);
 
-  ++auto_kf_size; // chain includes a terminator
-
   auto_key_frames = Z_Calloc(auto_kf_size, sizeof(auto_kf_t));
 
   auto_key_frames[0].prev = &auto_key_frames[auto_kf_size - 1];
@@ -202,6 +204,18 @@ void dsda_InitKeyFrame(void) {
   last_auto_kf = &auto_key_frames[auto_kf_size - 1];
 }
 
+void dsda_InitPlaybackKeyFrames() {
+  // Max of 60 keyframes are saved, and they need to have 1 minute in between each
+  playback_kf_size = demo_tics_count * demo_playerscount / TICRATE / 60;
+  if (playback_kf_size > 60)
+    playback_kf_size = 60;
+
+  if (playback_key_frames != NULL)
+    Z_Free(playback_key_frames);
+
+  playback_key_frames = Z_Calloc(playback_kf_size, sizeof(dsda_key_frame_t));
+}
+
 void dsda_ExportKeyFrame(byte* buffer, int length) {
   char name[40];
   int timestamp;
@@ -219,7 +233,7 @@ void dsda_ExportKeyFrame(byte* buffer, int length) {
 
 // Stripped down version of G_DoSaveGame
 void dsda_StoreKeyFrame(dsda_key_frame_t* key_frame, byte complete, byte export) {
-  key_frame->game_tic_count = logictic;
+  key_frame->game_tic_count = true_logictic;
 
   P_InitSaveBuffer();
 
@@ -413,3 +427,23 @@ void dsda_UpdateAutoKeyFrames(void) {
       dsda_CopyKeyFrame(&first_kf, current_key_frame);
   }
 }
+
+void dsda_UpdatePlaybackKeyFrames(void) {
+  int current_time;
+  int interval_tics;
+
+  if (gameaction != ga_nothing || playback_kf_size == 0) return;
+
+  current_time = totalleveltimes + leveltime;
+  interval_tics = (demo_tics_count * demo_playerscount) / playback_kf_size;
+
+  // Automatically save a key frame each interval
+  if (current_time % interval_tics == 0 &&
+      current_time / interval_tics < playback_kf_size) {
+
+    dsda_key_frame_t* current_key_frame = &playback_key_frames[current_time / interval_tics];
+
+    if (!current_key_frame->buffer)
+      dsda_StoreKeyFrame(current_key_frame, false, false);
+  }
+}
