@@ -43,16 +43,21 @@ typedef struct auto_kf_s {
 
 void dsda_StoreKeyFrame(dsda_key_frame_t* key_frame, byte complete, byte export);
 void dsda_RestoreKeyFrame(dsda_key_frame_t* key_frame, dboolean skip_wipe);
-void dsda_InitKeyFrame(void);
-void dsda_ContinueKeyFrame(void);
+dboolean dsda_RestoreClosestKeyFrame(int tic);
 int dsda_KeyFrameRestored(void);
+void dsda_ContinueKeyFrame(void);
+
+void dsda_InitAutoKeyFrames(void);
+void dsda_UpdateAutoKeyFrames(void);
+void dsda_ForgetAutoKeyFrames(void);
+void dsda_RewindAutoKeyFrame(void);
+void dsda_ResetAutoKeyFrameTimeout(void);
+
+void dsda_InitPlaybackKeyFrames(void);
+void dsda_UpdatePlaybackKeyFrames(void);
+
 void dsda_StoreTempKeyFrame(void);
 void dsda_StoreQuickKeyFrame(void);
 void dsda_RestoreQuickKeyFrame(void);
-dboolean dsda_RestoreClosestKeyFrame(int tic);
-void dsda_RewindAutoKeyFrame(void);
-void dsda_ResetAutoKeyFrameTimeout(void);
-void dsda_UpdateAutoKeyFrames(void);
-void dsda_ForgetAutoKeyFrames(void);
 
 #endif
