@@ -89,6 +89,8 @@ extern map_format_t map_format;
 int dsda_DoorType(int index);
 dboolean dsda_IsExitLine(int index);
 dboolean dsda_IsSecretExitLine(int index);
+dboolean dsda_IsDeathExitLine(int index);
+dboolean dsda_IsDeathSecretExitLine(int index);
 dboolean dsda_IsTeleportLine(int index);
 void dsda_ApplyZDoomMapFormat(void);
 void dsda_ApplyDefaultMapFormat(void);
