@@ -459,7 +459,7 @@ void dsda_DoomFDrawer(void) {
   else {
     // e6y: wide-res
     V_ClearBorder();
-    V_DrawNamePatch(0, 0, 0, end_data->end_pic, CR_DEFAULT, VPT_STRETCH);
+    V_DrawNamePatchFS(0, 0, 0, end_data->end_pic, CR_DEFAULT, VPT_STRETCH);
   }
 }
 
