@@ -18,8 +18,12 @@
 #ifndef __DSDA_PREFERENCES__
 #define __DSDA_PREFERENCES__
 
+#include "doomtype.h"
+
+void dsda_LoadWadPreferences(void);
 void dsda_HandleMapPreferences(void);
 void dsda_PreferOpenGL(void);
 void dsda_PreferSoftware(void);
+dboolean dsda_UseMapinfo(void);
 
 #endif
