@@ -27,6 +27,7 @@
 
 #include "dsda/args.h"
 #include "dsda/build.h"
+#include "dsda/demo.h"
 #include "dsda/features.h"
 #include "dsda/pause.h"
 #include "dsda/playback.h"
@@ -55,6 +56,9 @@ static void dsda_CacheSkipSetting(dboolean* old, dboolean* current) {
 static dboolean old_fastdemo, old_nodrawers, old_nosfxparm, old_nomusicparm;
 
 static void dsda_ApplySkipSettings(void) {
+  if (skip_mode)
+    return;
+
   dsda_CacheSkipSetting(&old_fastdemo, &fastdemo);
   dsda_CacheSkipSetting(&old_nodrawers, &nodrawers);
   dsda_CacheSkipSetting(&old_nosfxparm, &nosfxparm);
@@ -72,11 +76,11 @@ void dsda_EnterSkipMode(void) {
   extern void M_ClearMenus(void);
 
   dsda_TrackFeature(uf_skip);
+  dsda_ApplySkipSettings();
 
   skip_mode = true;
 
   M_ClearMenus();
-  dsda_ApplySkipSettings();
   dsda_ResetPauseMode();
   S_StopMusic();
   I_Init2();
@@ -146,7 +150,7 @@ void dsda_EvaluateSkipModeBuildTiccmd(void) {
         (
           demo_skiptics > 0 ?
             gametic > demo_skiptics :
-            dsda_PlaybackTics() - demo_skiptics >= demo_tics_count
+            dsda_DemoTic() - demo_skiptics >= demo_tics_count
         )
       ) ||
       (
