@@ -21,6 +21,7 @@
 #include "i_system.h"
 #include "lprintf.h"
 
+#include "dsda/args.h"
 #include "dsda/configuration.h"
 
 #include "time.h"
@@ -64,8 +65,10 @@ unsigned long long dsda_ElapsedTime(int timer) {
 
   clock_gettime(CLOCK_MONOTONIC, &now);
 
-  return (now.tv_nsec - dsda_time[timer].tv_nsec) / 1000 +
-         (now.tv_sec - dsda_time[timer].tv_sec) * 1000000;
+  return (unsigned long long) (
+           (signed long long) (now.tv_nsec - dsda_time[timer].tv_nsec) / 1000 +
+           (signed long long) (now.tv_sec - dsda_time[timer].tv_sec) * 1000000
+         );
 }
 
 unsigned long long dsda_ElapsedTimeMS(int timer) {
@@ -100,12 +103,16 @@ static void dsda_Throttle(int timer, unsigned long long target_time) {
 
 void dsda_LimitFPS(void) {
   extern int movement_smooth;
+  extern int window_focused;
 
+  int allow_limit;
   int fps_limit;
 
-  fps_limit = dsda_IntConfig(dsda_config_fps_limit);
+  allow_limit = (movement_smooth || !window_focused) && !dsda_Flag(dsda_arg_timedemo) && !dsda_Flag(dsda_arg_fastdemo);
+  fps_limit = window_focused ? dsda_IntConfig(dsda_config_fps_limit)
+                             : dsda_IntConfig(dsda_config_background_fps_limit);
 
-  if (movement_smooth && fps_limit) {
+  if (allow_limit && fps_limit) {
     unsigned long long target_time;
 
     target_time = 1000000 / fps_limit;
