@@ -104,9 +104,9 @@ void G_SkipDemoStartCheck(void);
 void G_SkipDemoCheck(void);
 int G_ReloadLevel(void);
 int G_GotoNextLevel(void);
+int G_GotoPrevLevel(void);
 
 void M_ChangeSkyMode(void);
-void M_ChangeMaxViewPitch(void);
 
 void M_ChangeFOV(void);
 
