@@ -490,7 +490,20 @@ void gld_EndAutomapDraw(void)
   glsl_PopNullShader();
 }
 
-void gld_DrawNumPatch_f(float x, float y, int lump, int cm, enum patch_translation_e flags)
+void gld_BeginMenuDraw(void)
+{
+  gld_InitColormapTextures(true);
+  glsl_PushNullShader();
+  gl_menu_lightmode_indexed = true;
+}
+
+void gld_EndMenuDraw(void)
+{
+  gl_menu_lightmode_indexed = false;
+  glsl_PopNullShader();
+}
+
+void gld_DrawNumPatch_f(float x, float y, int lump, dboolean center, int cm, enum patch_translation_e flags)
 {
   GLTexture *gltexture;
   float fU1,fU2,fV1,fV2;
@@ -530,8 +543,11 @@ void gld_DrawNumPatch_f(float x, float y, int lump, int cm, enum patch_translati
   }
 
   // [FG] automatically center wide patches without horizontal offset
-  if (gltexture->width > 320 && leftoffset == 0)
-    x -= (float)(gltexture->width - 320) / 2;
+  if (center)
+  {
+    if (gltexture->width > 320 && leftoffset == 0)
+      x -= (float)(gltexture->width - 320) / 2;
+  }
 
   if (flags & VPT_STRETCH_MASK)
   {
@@ -566,9 +582,9 @@ void gld_DrawNumPatch_f(float x, float y, int lump, int cm, enum patch_translati
   glEnd();
 }
 
-void gld_DrawNumPatch(int x, int y, int lump, int cm, enum patch_translation_e flags)
+void gld_DrawNumPatch(int x, int y, int lump, dboolean center, int cm, enum patch_translation_e flags)
 {
-  gld_DrawNumPatch_f((float)x, (float)y, lump, cm, flags);
+  gld_DrawNumPatch_f((float)x, (float)y, lump, center, cm, flags);
 }
 
 void gld_FillRaw(int lump, int x, int y, int src_width, int src_height, int dst_width, int dst_height, enum patch_translation_e flags)
@@ -666,7 +682,7 @@ void gld_FillPatch(int lump, int x, int y, int width, int height, enum patch_tra
 // use colormaps[0] as a fallback in such a case.
 const lighttable_t *gld_GetActiveColormap()
 {
-  if (V_IsAutomapLightmodeIndexed())
+  if (V_IsAutomapLightmodeIndexed() || V_IsMenuLightmodeIndexed())
     return colormaps[0];
   else if (fixedcolormap)
     return fixedcolormap;
@@ -718,7 +734,7 @@ void gld_DrawLine_f(float x0, float y0, float x1, float y1, int BaseColor)
   if (a == 0)
     return;
 
-  color = gld_LookupIndexedColor(BaseColor, V_IsUILightmodeIndexed() || V_IsAutomapLightmodeIndexed());
+  color = gld_LookupIndexedColor(BaseColor, V_IsUILightmodeIndexed() || V_IsAutomapLightmodeIndexed() || V_IsMenuLightmodeIndexed());
 
   line = M_ArrayGetNewItem(&map_lines, sizeof(line[0]));
 
@@ -826,7 +842,7 @@ void gld_DrawWeapon(int weaponlump, vissprite_t *vis, int lightlevel)
 
 void gld_FillBlock(int x, int y, int width, int height, int col)
 {
-  color_rgb_t color = gld_LookupIndexedColor(col, V_IsUILightmodeIndexed() || V_IsAutomapLightmodeIndexed());
+  color_rgb_t color = gld_LookupIndexedColor(col, V_IsUILightmodeIndexed() || V_IsAutomapLightmodeIndexed() || V_IsMenuLightmodeIndexed());
 
   glsl_PushNullShader();
 
@@ -850,12 +866,7 @@ void gld_FillBlock(int x, int y, int width, int height, int col)
 
 void gld_DrawShaded(int x, int y, int width, int height, int shade)
 {
-  // This is more messy than I'd like. (I hate this, but this is my current fix for the menu overlay invert)
-  // The `col` fixes the menu overlay from inverting during `invul_cm`.
-  // The 'automap` boolean is to undo the `col` invert for the automap.
-  dboolean automap = V_IsAutomapLightmodeIndexed();
-  int col = invul_cm && !automap ? 256 : 0;
-  color_rgb_t color = gld_LookupIndexedColor(col, V_IsUILightmodeIndexed() || V_IsAutomapLightmodeIndexed());
+  color_rgb_t color = gld_LookupIndexedColor(playpal_darkest, V_IsAutomapLightmodeIndexed() || V_IsMenuLightmodeIndexed());
 
   glsl_PushNullShader();
 
@@ -899,8 +910,8 @@ unsigned char *gld_ReadScreen(void)
 
   int src_row, dest_row, size, pixels_per_row;
 
-  pixels_per_row = gl_window_width * 3;
-  size = pixels_per_row * gl_window_height;
+  pixels_per_row = renderer_rect.w * 3;
+  size = pixels_per_row * renderer_rect.h;
   if (!scr || size > scr_size)
   {
     scr_size = size;
@@ -915,12 +926,12 @@ unsigned char *gld_ReadScreen(void)
     glPixelStorei(GL_PACK_ALIGNMENT, 1);
 
     glFlush();
-    glReadPixels(0, 0, gl_window_width, gl_window_height, GL_RGB, GL_UNSIGNED_BYTE, scr);
+    glReadPixels(0, 0, renderer_rect.w, renderer_rect.h, GL_RGB, GL_UNSIGNED_BYTE, scr);
 
     glPixelStorei(GL_PACK_ALIGNMENT, pack_aligment);
 
     // GL textures are bottom up, so copy the rows in reverse to flip vertically
-    for (src_row = gl_window_height - 1, dest_row = 0; src_row >= 0; --src_row, ++dest_row)
+    for (src_row = renderer_rect.h - 1, dest_row = 0; src_row >= 0; --src_row, ++dest_row)
     {
       memcpy(&buffer[dest_row * pixels_per_row],
               &scr[src_row * pixels_per_row],
@@ -1103,9 +1114,9 @@ void gld_EndDrawScene(void)
     glBegin(GL_TRIANGLE_STRIP);
     {
       glTexCoord2f(0.0f, 1.0f); glVertex2f(0.0f, 0.0f);
-      glTexCoord2f(0.0f, 0.0f); glVertex2f(0.0f, gl_window_height);
-      glTexCoord2f(1.0f, 1.0f); glVertex2f((float)gl_window_width, 0.0f);
-      glTexCoord2f(1.0f, 0.0f); glVertex2f((float)gl_window_width, (float)gl_window_height);
+      glTexCoord2f(0.0f, 0.0f); glVertex2f(0.0f, renderer_rect.h);
+      glTexCoord2f(1.0f, 1.0f); glVertex2f((float)renderer_rect.w, 0.0f);
+      glTexCoord2f(1.0f, 0.0f); glVertex2f((float)renderer_rect.w, (float)renderer_rect.h);
     }
     glEnd();
 
