@@ -46,6 +46,7 @@
 #include <SDL.h>
 #include <math.h>
 
+#include "doomtype.h"
 #include "doomstat.h"
 #include "v_video.h"
 #include "gl_intern.h"
@@ -70,7 +71,7 @@ typedef struct
 
 typedef struct
 {
-  int id;
+  GLuint id;
   int rows, columns;
   int loopcount;
   GLSkyLoopDef *loops;
@@ -173,7 +174,7 @@ void gld_AddSkyTexture(GLWall *wall, int sky1, int sky2, int skytype)
       wall->skyyaw = (double) (viewangle + s->textureoffset) / (double) ANGLE_MAX;
       wall->skypitch = skyYShift;
       wall->skyoffset = (((float)s->rowoffset/(float)FRACUNIT - 28.0f)/wall->gltexture->buffer_height);
-      wall->flag = l->special == 282 ? GLDWF_SKY : GLDWF_SKYFLIP;
+      wall->flag = l->special == 272 ? GLDWF_SKY : GLDWF_SKYFLIP;
     }
   }
   else
