@@ -21,6 +21,7 @@
 ========================
 */
 
+#include "am_map.h"
 #include "doomstat.h"
 #include "d_event.h"
 #include "s_sound.h"
@@ -170,8 +171,8 @@ static void IN_DrawInterpic(void)
   snprintf(name, sizeof(name), "MAPE%d", gameepisode);
 
   // e6y: wide-res
-  V_FillBorder(-1, 0);
-  V_DrawNamePatch(0, 0, 0, name, CR_DEFAULT, VPT_STRETCH);
+  V_ClearBorder();
+  V_DrawNamePatchFS(0, 0, 0, name, CR_DEFAULT, VPT_STRETCH);
 }
 
 static void IN_DrawBeenThere(int i)
@@ -220,8 +221,6 @@ static void IN_InitVariables(wbstartstruct_t* wbstartstruct)
 //
 //========================================================================
 
-extern void AM_Stop(void);
-
 void IN_Start(wbstartstruct_t* wbstartstruct)
 {
     V_SetPalette(0);
@@ -233,7 +232,7 @@ void IN_Start(wbstartstruct_t* wbstartstruct)
     skipintermission = false;
     intertime = 0;
     oldintertime = 0;
-    AM_Stop();
+    AM_Stop(false);
     S_ChangeMusic(heretic_mus_intr, true);
 }
 
@@ -577,7 +576,7 @@ void IN_Drawer(void)
 void IN_DrawStatBack(void)
 {
     // e6y: wide-res
-    V_FillBorder(-1, 0);
+    V_ClearBorder();
     V_DrawBackground("FLOOR16", 0);
 }
 
