@@ -31,7 +31,8 @@ void MN_DrawLoad(void);
 void MN_DrawSave(void);
 void MN_DrawPause(void);
 void MN_DrawMessage(const char* messageString);
-void MN_DrawSlider(int x, int y, int width, int slot);
+void MN_DrawSlider(int x, int y, int width, int range, int slot);
+void MN_DrawTitle(int y, const char *text, int cm);
 void MN_DrTextA(const char *text, int x, int y);
 int MN_TextAHeight(const char *text);
 int MN_TextAWidth(const char *text);
