@@ -75,23 +75,23 @@
 //
 //-----------------------------------------------------------------------------
 
-static void cheat_mus();
+static void cheat_mus(char buf[3]);
 static void cheat_choppers();
 static void cheat_god();
 static void cheat_fa();
 static void cheat_k();
 static void cheat_kfa();
 static void cheat_noclip();
-static void cheat_pw();
+static void cheat_pw(int pw);
 static void cheat_behold();
 static void cheat_clev0();
-static void cheat_clev();
+static void cheat_clev(char buf[3]);
 static void cheat_mypos();
 static void cheat_rate();
 static void cheat_comp0();
-static void cheat_comp();
+static void cheat_comp(char buf[3]);
 static void cheat_skill0();
-static void cheat_skill();
+static void cheat_skill(char buf[1]);
 static void cheat_friction();
 static void cheat_pushers();
 static void cheat_massacre();
@@ -103,11 +103,11 @@ static void cheat_hom();
 static void cheat_fast();
 static void cheat_tntkey();
 static void cheat_tntkeyx();
-static void cheat_tntkeyxx();
+static void cheat_tntkeyxx(int key);
 static void cheat_tntweap();
-static void cheat_tntweapx();
+static void cheat_tntweapx(char buf[3]);
 static void cheat_tntammo();
-static void cheat_tntammox();
+static void cheat_tntammox(char buf[1]);
 static void cheat_smart();
 static void cheat_pitch();
 static void cheat_megaarmour();
@@ -120,14 +120,14 @@ static void cheat_fly();
 static void cheat_reset_health();
 static void cheat_tome();
 static void cheat_chicken();
-static void cheat_artifact();
+static void cheat_artifact(char buf[3]);
 
 // hexen
 static void cheat_inventory();
 static void cheat_puzzle();
-static void cheat_class();
+static void cheat_class(char buf[2]);
 static void cheat_init();
-static void cheat_script();
+static void cheat_script(char buf[3]);
 
 //-----------------------------------------------------------------------------
 //
@@ -260,8 +260,7 @@ cheatseq_t cheat[] = {
 
 //-----------------------------------------------------------------------------
 
-static void cheat_mus(buf)
-char buf[3];
+static void cheat_mus(char buf[3])
 {
   int musnum, muslump;
   int epsd, map;
@@ -579,13 +578,29 @@ static void cheat_comp(char buf[3])
   }
 }
 
+// Get skill strings
+static const char* dsda_skill_str(void)
+{
+  if (hexen)
+  {
+    if (PlayerClass[consoleplayer] == PCLASS_FIGHTER)
+        return hexen_skill_fighter[gameskill];
+    else if (PlayerClass[consoleplayer] == PCLASS_CLERIC)
+        return hexen_skill_cleric[gameskill];
+    else if (PlayerClass[consoleplayer] == PCLASS_MAGE)
+        return hexen_skill_mage[gameskill];
+  }
+
+  return skill_infos[gameskill].name;
+}
+
 // Check skill cheat
 static void cheat_skill0()
 {
-  if (raven || tc_game)
-    return doom_printf("Skill: %i", gameskill + 1);
-
-  doom_printf("Skill: %i - %s", gameskill + 1, skill_str[gameskill + 1]);
+  if (!tc_game)
+    doom_printf("Skill: %i - %s", gameskill + 1, dsda_skill_str());
+  else
+    doom_printf("Skill: %i", gameskill + 1);
 }
 
 // Skill cheat
@@ -593,14 +608,14 @@ static void cheat_skill(char buf[1])
 {
   int skill = buf[0] - '0';
 
-  if (skill >= 1 && skill <= 5)
+  if (skill >= 1 && skill <= num_skills)
   {
     gameskill = skill - 1;
 
-    if (raven || tc_game)
-      doom_printf("Next Level Skill: %i", gameskill + 1);
+    if (!tc_game)
+      doom_printf("Next Level Skill: %i - %s", gameskill + 1, dsda_skill_str());
     else
-      doom_printf("Next Level Skill: %i - %s", gameskill + 1, skill_str[gameskill + 1]);
+      doom_printf("Next Level Skill: %i", gameskill + 1);
 
     dsda_UpdateGameSkill(gameskill);
   }
@@ -819,8 +834,7 @@ static void cheat_tntweap()
   dsda_AddMessage(gamemode == commercial ? "Weapon number 1-9" : "Weapon number 1-8");
 }
 
-static void cheat_tntweapx(buf)
-char buf[3];
+static void cheat_tntweapx(char buf[3])
 {
   int w = *buf - '1';
 
@@ -849,8 +863,7 @@ static void cheat_tntammo()
   dsda_AddMessage("Ammo 1-4, Backpack");
 }
 
-static void cheat_tntammox(buf)
-char buf[1];
+static void cheat_tntammox(char buf[1])
 {
   int a = *buf - '1';
   if (*buf == 'b')  // Ty 03/27/98 - strings *not* externalized
