@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C -*-
  *-----------------------------------------------------------------------------
  *
  *
@@ -42,7 +42,9 @@
 #include "lprintf.h"
 #include "e6y.h"//e6y
 
+#include "dsda/id_list.h"
 #include "dsda/map_format.h"
+#include "dsda/messenger.h"
 
 #include "hexen/p_acs.h"
 #include "hexen/sn_sonix.h"
@@ -79,26 +81,26 @@ void T_VerticalCompatibleDoor(vldoor_t *door)
           case blazeRaise:
           case genBlazeRaise:
             door->direction = -1; // time to go back down
-            S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
+            S_StartSectorSound(door->sector, sfx_bdcls);
             break;
 
           case normal:
           case genRaise:
           case vld_normal:
             door->direction = -1; // time to go back down
-            S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_dorcls);
+            S_StartSectorSound(door->sector, g_sfx_dorcls);
             break;
 
           case close30ThenOpen:
           case genCdO:
           case vld_close30ThenOpen:
             door->direction = 1;  // time to go back up
-            S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_doropn);
+            S_StartSectorSound(door->sector, g_sfx_doropn);
             break;
 
           case genBlazeCdO:
             door->direction = 1;  // time to go back up
-            S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
+            S_StartSectorSound(door->sector, sfx_bdopn);
             break;
 
           default:
@@ -117,13 +119,13 @@ void T_VerticalCompatibleDoor(vldoor_t *door)
           case vld_raiseIn5Mins:
             door->direction = 1;  // time to raise then
             door->type = g_door_normal; // door acts just like normal 1 DR door now
-            S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_doropn);
+            S_StartSectorSound(door->sector, g_sfx_doropn);
             break;
 
           case waitCloseDoor:
             door->direction = -1;
             door->type = closeDoor;
-            S_StartSound((mobj_t *)&door->sector->soundorg, g_sfx_dorcls);
+            S_StartSectorSound(door->sector, g_sfx_dorcls);
 
           default:
             break;
@@ -172,7 +174,7 @@ void T_VerticalCompatibleDoor(vldoor_t *door)
             P_RemoveThinker (&door->thinker);  // unlink and free
             // killough 4/15/98: remove double-closing sound of blazing doors
             if (comp[comp_blazing])
-              S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
+              S_StartSectorSound(door->sector, sfx_bdcls);
             break;
 
           case normal:
@@ -187,7 +189,7 @@ void T_VerticalCompatibleDoor(vldoor_t *door)
           case vld_close:
             door->sector->ceilingdata = NULL;
             P_RemoveThinker(&door->thinker);        // unlink and free
-            S_StartSound((mobj_t *)&door->sector->soundorg, g_sfx_dorlnd);
+            S_StartSectorSound(door->sector, g_sfx_dorlnd);
             break;
 
           // close then open doors start waiting
@@ -234,14 +236,14 @@ void T_VerticalCompatibleDoor(vldoor_t *door)
           case genBlazeRaise:
             door->direction = 1;
             if (!comp[comp_blazing]) {
-              S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
+              S_StartSectorSound(door->sector, sfx_bdopn);
               break;
             }
             // fallthrough
 
           default:             // other types bounce off the obstruction
             door->direction = 1;
-            S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_doropn);
+            S_StartSectorSound(door->sector, g_sfx_doropn);
             break;
         }
       }
@@ -464,8 +466,8 @@ int EV_DoLockedDoor
     case 133:
       if (!p->cards[it_bluecard] && !p->cards[it_blueskull])
       {
-        p->message = s_PD_BLUEO;             // Ty 03/27/98 - externalized
-        S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
+        dsda_AddPlayerMessage(s_PD_BLUEO, p);
+        S_StartMobjSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
       break;
@@ -474,8 +476,8 @@ int EV_DoLockedDoor
     case 135:
       if (!p->cards[it_redcard] && !p->cards[it_redskull])
       {
-        p->message = s_PD_REDO;              // Ty 03/27/98 - externalized
-        S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
+        dsda_AddPlayerMessage(s_PD_REDO, p);
+        S_StartMobjSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
       break;
@@ -484,8 +486,8 @@ int EV_DoLockedDoor
     case 137:
       if (!p->cards[it_yellowcard] && !p->cards[it_yellowskull])
       {
-        p->message = s_PD_YELLOWO;           // Ty 03/27/98 - externalized
-        S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
+        dsda_AddPlayerMessage(s_PD_YELLOWO, p);
+        S_StartMobjSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
       break;
@@ -508,22 +510,20 @@ int EV_DoDoor
 ( line_t* line,
   vldoor_e  type )
 {
-  int   secnum,rtn;
+  const int *id_p;
+  int rtn;
   sector_t* sec;
   vldoor_t* door;
 
-  secnum = -1;
   rtn = 0;
 
-  if (ProcessNoTagLines(line, &sec, &secnum)) {if (zerotag_manual) goto manual_door; else {return rtn;}};//e6y
   // open all doors with the same tag as the activating line
-  while ((secnum = P_FindSectorFromLineTag(line,secnum)) >= 0)
+  FIND_SECTORS(id_p, line->tag)
   {
-    sec = &sectors[secnum];
-manual_door://e6y
+    sec = &sectors[*id_p];
     // if the ceiling already moving, don't start the door action
-    if (P_CeilingActive(sec)) { //jff 2/22/98
-      if (!zerotag_manual) continue; else {return rtn;}}; //e6y
+    if (P_CeilingActive(sec)) //jff 2/22/98
+      continue;
 
     // new door thinker
     rtn = 1;
@@ -548,7 +548,7 @@ manual_door://e6y
         door->topheight -= 4*FRACUNIT;
         door->direction = -1;
         door->speed = VDOORSPEED * 4;
-        S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
+        S_StartSectorSound(door->sector, sfx_bdcls);
         break;
 
       case closeDoor:
@@ -556,14 +556,14 @@ manual_door://e6y
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
         door->direction = -1;
-        S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_dorcls);
+        S_StartSectorSound(door->sector, g_sfx_dorcls);
         break;
 
       case close30ThenOpen:
       case vld_close30ThenOpen:
         door->topheight = sec->ceilingheight;
         door->direction = -1;
-        S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_dorcls);
+        S_StartSectorSound(door->sector, g_sfx_dorcls);
         break;
 
       case blazeRaise:
@@ -573,7 +573,7 @@ manual_door://e6y
         door->topheight -= 4*FRACUNIT;
         door->speed = VDOORSPEED * 4;
         if (door->topheight != sec->ceilingheight)
-          S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
+          S_StartSectorSound(door->sector, sfx_bdopn);
         break;
 
       case vld_normal_turbo:
@@ -589,13 +589,12 @@ manual_door://e6y
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
         if (door->topheight != sec->ceilingheight)
-          S_StartSound((mobj_t *)&door->sector->soundorg,g_sfx_doropn);
+          S_StartSectorSound(door->sector, g_sfx_doropn);
         break;
 
       default:
         break;
     }
-    if (zerotag_manual) return rtn; //e6y
   }
   return rtn;
 }
@@ -633,8 +632,8 @@ int EV_VerticalDoor
         return 0;
       if (!player->cards[it_bluecard] && !player->cards[it_blueskull])
       {
-          player->message = s_PD_BLUEK;         // Ty 03/27/98 - externalized
-          S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
+          dsda_AddPlayerMessage(s_PD_BLUEK, player);
+          S_StartMobjSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
@@ -645,8 +644,8 @@ int EV_VerticalDoor
           return 0;
       if (!player->cards[it_yellowcard] && !player->cards[it_yellowskull])
       {
-          player->message = s_PD_YELLOWK;       // Ty 03/27/98 - externalized
-          S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
+          dsda_AddPlayerMessage(s_PD_YELLOWK, player);
+          S_StartMobjSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
@@ -657,8 +656,8 @@ int EV_VerticalDoor
           return 0;
       if (!player->cards[it_redcard] && !player->cards[it_redskull])
       {
-          player->message = s_PD_REDK;          // Ty 03/27/98 - externalized
-          S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
+          dsda_AddPlayerMessage(s_PD_REDK, player);
+          S_StartMobjSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
@@ -670,7 +669,7 @@ int EV_VerticalDoor
   // if the wrong side of door is pushed, give oof sound
   if (line->sidenum[1]==NO_INDEX)                     // killough
   {
-    S_StartSound(player->mo,sfx_oof);           // killough 3/20/98
+    S_StartMobjSound(player->mo,sfx_oof);           // killough 3/20/98
     return 0;
   }
 
@@ -744,11 +743,11 @@ int EV_VerticalDoor
   {
     case 117: // blazing door raise
     case 118: // blazing door open
-      S_StartSound((mobj_t *)&sec->soundorg,sfx_bdopn);
+      S_StartSectorSound(sec, sfx_bdopn);
       break;
 
     default:  // normal or locked door sound
-      S_StartSound((mobj_t *)&sec->soundorg,sfx_doropn);
+      S_StartSectorSound(sec, sfx_doropn);
       break;
   }
 
@@ -906,7 +905,7 @@ void Heretic_EV_VerticalDoor(line_t * line, mobj_t * thing)
             if (!player->cards[key_blue])
             {
                 P_SetMessage(player, HERETIC_TXT_NEEDBLUEKEY, false);
-                S_StartSound(NULL, heretic_sfx_plroof);
+                S_StartVoidSound(heretic_sfx_plroof);
                 return;
             }
             break;
@@ -919,7 +918,7 @@ void Heretic_EV_VerticalDoor(line_t * line, mobj_t * thing)
             if (!player->cards[key_yellow])
             {
                 P_SetMessage(player, HERETIC_TXT_NEEDYELLOWKEY, false);
-                S_StartSound(NULL, heretic_sfx_plroof);
+                S_StartVoidSound(heretic_sfx_plroof);
                 return;
             }
             break;
@@ -932,7 +931,7 @@ void Heretic_EV_VerticalDoor(line_t * line, mobj_t * thing)
             if (!player->cards[key_green])
             {
                 P_SetMessage(player, HERETIC_TXT_NEEDGREENKEY, false);
-                S_StartSound(NULL, heretic_sfx_plroof);
+                S_StartVoidSound(heretic_sfx_plroof);
                 return;
             }
             break;
@@ -965,7 +964,7 @@ void Heretic_EV_VerticalDoor(line_t * line, mobj_t * thing)
         }
     }
 
-    S_StartSound(&sec->soundorg, heretic_sfx_doropn);
+    S_StartSectorSound(sec, heretic_sfx_doropn);
 
     //
     // new door thinker
@@ -1008,7 +1007,7 @@ void Heretic_EV_VerticalDoor(line_t * line, mobj_t * thing)
 // hexen
 
 static void P_SpawnZDoomDoor(sector_t *sec, vldoor_e type, line_t *line, fixed_t speed,
-                             int topwait, byte lightTag, int topcountdown)
+                             int topwait, int lightTag, int topcountdown)
 {
   vldoor_t *door;
 
@@ -1032,20 +1031,20 @@ static void P_SpawnZDoomDoor(sector_t *sec, vldoor_e type, line_t *line, fixed_t
       door->topheight = P_FindLowestCeilingSurrounding(sec);
       door->topheight -= 4 * FRACUNIT;
       door->direction = -1;
-      S_StartSound((mobj_t *) &door->sector->soundorg, g_sfx_dorcls);
+      S_StartSectorSound(door->sector, g_sfx_dorcls);
       break;
     case genCdO:
       door->topheight = sec->ceilingheight;
       door->direction = -1;
       door->topwait = topwait;
-      S_StartSound((mobj_t *) &door->sector->soundorg, g_sfx_dorcls);
+      S_StartSectorSound(door->sector, g_sfx_dorcls);
       break;
     case normal:
     case openDoor:
       door->direction = 1;
       door->topheight = P_FindLowestCeilingSurrounding(sec);
       door->topheight -= 4 * FRACUNIT;
-      S_StartSound((mobj_t *) &door->sector->soundorg, g_sfx_doropn);
+      S_StartSectorSound(door->sector, g_sfx_doropn);
       break;
     case waitRaiseDoor:
       door->direction = 2;
@@ -1062,16 +1061,15 @@ static void P_SpawnZDoomDoor(sector_t *sec, vldoor_e type, line_t *line, fixed_t
   }
 }
 
-int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, byte tag, byte speed_byte, int topwait,
-                   zdoom_lock_t lock, byte lightTag, dboolean boomgen, int topcountdown)
+int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, int tag, fixed_t speed, int topwait,
+                   zdoom_lock_t lock, int lightTag, dboolean boomgen, int topcountdown)
 {
   sector_t *sec;
   vldoor_t *door;
-  fixed_t speed;
 
-  speed = (fixed_t) speed_byte * FRACUNIT / 8;
+  speed *= FRACUNIT / 8;
 
-  if (lock && !P_CanUnlockZDoomDoor(mo->player, lock))
+  if (lock && !P_CheckKeys(mo, lock, true))
     return 0;
 
   if (!tag)
@@ -1083,7 +1081,7 @@ int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, byte tag, byte speed
     if (line->sidenum[1] == NO_INDEX)
     {
       if (mo->player) // is this check necessary?
-        S_StartSound(mo, sfx_oof);
+        S_StartMobjSound(mo, sfx_oof);
       return 0;
     }
 
@@ -1107,10 +1105,10 @@ int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, byte tag, byte speed
           {
             door->direction = 1;
 
-            S_StartSound((mobj_t *) &door->sector->soundorg, g_sfx_doropn);
+            S_StartSectorSound(door->sector, g_sfx_doropn);
             return 1;
           }
-          else if (!(line->flags & ML_SPAC_PUSH))
+          else if (!(line->activation & (SPAC_PUSH | SPAC_MPUSH)))
             // [RH] activate push doors don't go back down when you
             //    run into them (otherwise opening them would be
             //    a real pain).
@@ -1120,7 +1118,7 @@ int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, byte tag, byte speed
 
             door->direction = -1; // start going down immediately
 
-            S_StartSound((mobj_t *) &door->sector->soundorg, g_sfx_dorcls);
+            S_StartSectorSound(door->sector, g_sfx_dorcls);
             return 1;
           }
           else
@@ -1138,12 +1136,12 @@ int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, byte tag, byte speed
   }
   else
   {
-    int secnum = -1;
+    const int *id_p;
     int retcode = 0;
 
-    while ((secnum = P_FindSectorFromTag(tag, secnum)) >= 0)
+    FIND_SECTORS(id_p, tag)
     {
-      sec = &sectors[secnum];
+      sec = &sectors[*id_p];
       if (sec->ceilingdata)
       {
         continue;
@@ -1158,18 +1156,17 @@ int EV_DoZDoomDoor(vldoor_e type, line_t *line, mobj_t *mo, byte tag, byte speed
 
 int Hexen_EV_DoDoor(line_t * line, byte * args, vldoor_e type)
 {
-    int secnum;
+    const int *id_p;
     int retcode;
     sector_t *sec;
     vldoor_t *door;
     fixed_t speed;
 
     speed = args[1] * FRACUNIT / 8;
-    secnum = -1;
     retcode = 0;
-    while ((secnum = P_FindSectorFromTag(args[0], secnum)) >= 0)
+    FIND_SECTORS(id_p, args[0])
     {
-        sec = &sectors[secnum];
+        sec = &sectors[*id_p];
         if (sec->floordata || sec->ceilingdata)
         {
             continue;
@@ -1205,7 +1202,7 @@ int Hexen_EV_DoDoor(line_t * line, byte * args, vldoor_e type)
         }
         door->type = type;
         door->speed = speed;
-        door->topwait = args[2];        // line->arg3
+        door->topwait = args[2];
         SN_StartSequence((mobj_t *) & door->sector->soundorg,
                          SEQ_DOOR_STONE + door->sector->seqType);
     }
@@ -1251,8 +1248,8 @@ dboolean Hexen_EV_VerticalDoor(line_t * line, mobj_t * thing)
             door->type = DREV_NORMAL;
             break;
     }
-    door->speed = line->arg2 * (FRACUNIT / 8);
-    door->topwait = line->arg3;
+    door->speed = line->special_args[1] * (FRACUNIT / 8);
+    door->topwait = line->special_args[2];
 
     //
     // find the top and bottom of the movement range
