@@ -143,4 +143,10 @@ void R_ResetColumnBuffer(void);
 void R_SetFuzzPos(int fuzzpos);
 int R_GetFuzzPos();
 
+// height is the height of the last column, in pixels
+void R_ResetFuzzCol(int height);
+
+// Calls R_ResetFuzzCol if x is aligned to the fuzz cell grid
+void R_CheckFuzzCol(int x, int height);
+
 #endif
