@@ -63,6 +63,7 @@
 
 #include "dsda/configuration.h"
 #include "dsda/exhud.h"
+#include "dsda/features.h"
 #include "dsda/map_format.h"
 #include "dsda/mapinfo.h"
 #include "dsda/render_stats.h"
@@ -764,6 +765,9 @@ void R_ExecuteSetViewSize (void)
     }
   }
 
+  I_SetWindowRect();
+  I_SetViewportRect();
+
   if (V_IsOpenGLMode())
     dsda_GLSetRenderViewportParams();
 
@@ -927,11 +931,20 @@ static void R_SetupFrame (player_t *player)
   int i, cm;
 
   int FocalTangent = finetangent[FINEANGLES/4 + FieldOfView/2];
+  int extra_brightness = dsda_IntConfig(dsda_config_extra_level_brightness);
 
   viewplayer = player;
 
   extralight = player->extralight;
 
+  if (extra_brightness < 0 || extra_brightness > 4) {
+    extra_brightness = 0;
+  }
+  if (extra_brightness != 0) {
+      dsda_TrackFeature(uf_levelbrightness);
+  }
+  extralight += extra_brightness;
+
   viewsin = finesine[viewangle>>ANGLETOFINESHIFT];
   viewcos = finecosine[viewangle>>ANGLETOFINESHIFT];
 
