@@ -206,8 +206,8 @@ void R_FixWiggle(sector_t *sec)
 
 static fixed_t R_ScaleFromGlobalAngle(angle_t visangle)
 {
-  int anglea = ANG90 + (visangle - viewangle);
-  int angleb = ANG90 + (visangle - rw_normalangle);
+  angle_t anglea = ANG90 + (visangle - viewangle);
+  angle_t angleb = ANG90 + (visangle - rw_normalangle);
   int den = FixedMul(rw_distance, finesine[anglea >> ANGLETOFINESHIFT]);
   // proff 11/06/98: Changed for high-res
   fixed_t num = FixedMul(projectiony, finesine[angleb >> ANGLETOFINESHIFT]);
