@@ -68,7 +68,11 @@ void S_StartSectorSound(sector_t *sector, int sfx_id);
 void S_LoopSectorSound(sector_t *sector, int sfx_id, int timeout);
 
 void S_StartMobjSound(mobj_t *mobj, int sfx_id);
+void S_LoopMobjSound(mobj_t *mobj, int sfx_id, int timeout);
+
 void S_StartVoidSound(int sfx_id);
+void S_LoopVoidSound(int sfx_id, int timeout);
+
 void S_StartLineSound(line_t *line, degenmobj_t *soundorg, int sfx_id);
 
 // Will start a sound at a given volume.
@@ -91,6 +95,7 @@ void S_StartMusic(int music_id);
 // Start music using <music_id> from sounds.h, and set whether looping
 void S_ChangeMusic(int music_id, int looping);
 void S_ChangeMusInfoMusic(int lumpnum, int looping);
+dboolean S_ChangeMusicByName(const char *name, dboolean looping);
 void S_RestartMusic(void);
 
 // Stops the music fer sure.
@@ -100,6 +105,10 @@ void S_StopMusic(void);
 void S_PauseSound(void);
 void S_ResumeSound(void);
 
+void S_AdjustAttenuation(float attenuation);
+void S_AdjustVolume(float volume);
+void S_ResetAdjustments(void);
+
 //
 // Updates music & sounds
 //
@@ -125,4 +134,6 @@ void S_StartSongName(const char *songLump, dboolean loop);
 dboolean S_GetSoundPlayingInfo(void * mobj, int sound_id);
 int S_GetSoundID(const char *name);
 
+void S_ResetVolume(void);
+
 #endif
