@@ -249,7 +249,7 @@ bool Scanner::GetNextToken(bool expandState)
 
 	char cur = data[scanPos++];
 	// Determine by first character
-	if(cur == '_' || (cur >= 'A' && cur <= 'Z') || (cur >= 'a' && cur <= 'z'))
+	if(cur == '_' || cur == '$' || (cur >= 'A' && cur <= 'Z') || (cur >= 'a' && cur <= 'z'))
 		nextState.token = TK_Identifier;
 	else if(cur >= '0' && cur <= '9')
 	{
@@ -325,7 +325,7 @@ bool Scanner::GetNextToken(bool expandState)
 				default:
 					break;
 				case TK_Identifier:
-					if(cur != '_' && (cur < 'A' || cur > 'Z') && (cur < 'a' || cur > 'z') && (cur < '0' || cur > '9'))
+					if(cur != '_' && (cur < 'A' || cur > 'Z') && (cur < 'a' || cur > 'z') && (cur < '0' || cur > '9') && cur != '/' && cur != '\\')
 						end = scanPos;
 					break;
 				case TK_IntConst:
@@ -396,6 +396,9 @@ bool Scanner::GetNextToken(bool expandState)
 			else
 				break;
 		}
+
+		if (start == end && scanPos == length)
+			end = scanPos;
 	}
 
 	if(end-start > 0 || stringFinished)
@@ -454,6 +457,11 @@ void Scanner::IncrementLine()
 	lineStart = scanPos;
 }
 
+void Scanner::SkipLine()
+{
+	while (tokenLine == line && GetNextToken());
+}
+
 void Scanner::Error(int token)
 {
 	if (token < TK_NumSpecialTokens && this->token >= TK_Identifier && this->token < TK_NumSpecialTokens)
@@ -591,6 +599,16 @@ bool Scanner::CheckFloat()
 	return res;
 }
 
+bool Scanner::CheckString()
+{
+	return CheckToken(TK_StringConst) || CheckToken(TK_Identifier);
+}
+
+bool Scanner::StringMatch(const char *target)
+{
+	return !strcmpnocase(string, target);
+}
+
 void Scanner::MustGetInteger()
 {
 	if (!ScanInteger()) Error(TK_IntConst);
@@ -601,6 +619,14 @@ void Scanner::MustGetFloat()
 	if (!ScanFloat()) Error(TK_FloatConst);
 }
 
+void Scanner::MustGetString()
+{
+	if (!CheckString())
+	{
+		ErrorF("Expected String Constant or Identifier");
+		return;
+	}
+}
 
 bool Scanner::TokensLeft() const
 {
