@@ -164,12 +164,14 @@ char*   W_ReadLumpToString (int lump);
 const void* W_SafeLumpByNum (int lump);
 const void* W_LumpByNum (int lump);
 const void* W_LockLumpNum(int lump);
+void *W_GetModifiableLumpData(int lump);
 
 int W_LumpNumExists(int lump);
 int W_LumpNameExists(const char *name);
 int W_LumpNameExists2(const char *name, int ns);
 int W_PWADLumpNumExists(int lump);
 int W_PWADLumpNameExists(const char *name);
+int W_PWADMapExists(void);
 
 // CPhipps - convenience macros
 //#define W_LumpByNum(num) (W_LumpByNum)((num),1)
